@@ -5,12 +5,12 @@
     This module implements a simple JavaScript serializer.
     Uses the basestring encode function from simplejson by Bob Ippolito.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-from typing import Any, Dict, IO, List, Match, Union
+from typing import IO, Any, Dict, List, Match, Union
 
 _str_re = re.compile(r'"(\\\\|\\"|[^"])*"')
 _int_re = re.compile(r'\d+')
@@ -83,7 +83,7 @@ def dumps(obj: Any, key: bool = False) -> str:
     if obj is None:
         return 'null'
     elif obj is True or obj is False:
-        return obj and 'true' or 'false'
+        return 'true' if obj else 'false'
     elif isinstance(obj, (int, float)):
         return str(obj)
     elif isinstance(obj, dict):
